﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics.Eventing.Reader;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DataManager
{
    public partial class UIFund : Form
    {
        private int userId;
        private string fundId = string.Empty;

        private DataTable fundInfo = new DataTable();

        private DataTable tbNav;

        public UIFund(string fundId, int userId)
        {
            this.fundId = fundId;
            this.userId = userId;
            
            InitializeComponent();
            InitializeData();
        }

        public string FundId { get { return fundId; } }

        public int UserId { get { return userId; } }

        private void InitializeData()
        { 
            txtFundId.Text = fundId;

            if (fundId != string.Empty)
            {

                fundInfo = DataAccess.Get_dm_fund_information(fundId, null);

                #region Basic Info

                if (fundInfo == null || fundInfo.Rows.Count <= 0)
                {
                    lblFundName.Text = string.Format("基金{0}不存在，得跟管理员说一下", fundId);
                    lblFundName.ForeColor = Color.Red;
                    return;
                }
                lblFundName.Text = fundInfo.Rows[0]["fund_name"].ToString();
                txtFundId.ReadOnly = true;
                txtRegisterCode.Text = fundInfo.Rows[0]["register_number"].ToString();
                txtRegisterCode.ReadOnly = true;
                lblInceptionDate.Text = fundInfo.Rows[0]["inception_date"].ToString();

                string s = fundInfo.Rows[0]["fund_type"].ToString();
                if (s != string.Empty)
                    lblFundType.Text = UIConstants.FundType[int.Parse(s)];
                else
                    lblFundType.Text = "";

                lblCompany.Text = fundInfo.Rows[0]["company_name"].ToString();
                lblCompany.ForeColor = Color.DodgerBlue;

                lblRegisteDate.Text = fundInfo.Rows[0]["register_date"].ToString();
                
                s = fundInfo.Rows[0]["manager_type"].ToString();
                if (s != string.Empty)
                    lblManagementType.Text = UIConstants.ManagerType[int.Parse(s)];
                else
                    lblManagementType.Text = "";

                s = fundInfo.Rows[0]["base_currency"].ToString();
                if (s != string.Empty)
                    lblCurrency.Text = UIConstants.Currency[int.Parse(s)];
                else
                    lblCurrency.Text = "";

                s = fundInfo.Rows[0]["fund_status"].ToString();
                if (s != String.Empty)
                    lblStatus.Text = UIConstants.FundStatus[int.Parse(s)];
                else
                    lblStatus.Text = "";

                lblCustodian.Text = fundInfo.Rows[0]["custodian_name"].ToString();
                lblAmacUpdateDate.Text = fundInfo.Rows[0]["zjx_last_info_update_time"].ToString();
                
                txtFundShortName.Text = fundInfo.Rows[0]["fund_short_name"].ToString();
                
                
                cmbStrategy.DataSource = new BindingSource(UIConstants.HedgeFundStrategy, null); ;
                cmbStrategy.DisplayMember = "Value";
                cmbStrategy.ValueMember = "Key";
                s = fundInfo.Rows[0]["strategy"].ToString();
                int strategyId = -1;
                if (s != string.Empty) strategyId = int.Parse(s);
                cmbStrategy.SelectedValue = strategyId;

                cmbSubStrategy.DataSource = new BindingSource(LoadSubStrategy(strategyId), null);
                cmbSubStrategy.DisplayMember = "Value";
                cmbSubStrategy.ValueMember = "Key";
                s = fundInfo.Rows[0]["substrategy"].ToString();
                int substrategyId = -1;
                if (s != string.Empty) substrategyId = int.Parse(s);
                cmbSubStrategy.SelectedValue = substrategyId;

                // 净值频率
                string navFreq = fundInfo.Rows[0]["nav_frequency"].ToString().Trim();
                if (string.IsNullOrEmpty(navFreq)) navFreq = "未知";
                cmbNavFrequency.SelectedIndex = cmbNavFrequency.Items.IndexOf(navFreq);

                chkRating.Checked = fundInfo.Rows[0]["is_rating"].ToString() == "1" ? true : false;
                chkRanking.Checked = fundInfo.Rows[0]["is_ranking"].ToString() == "1" ? true : false;
                chkAuthorized.Checked = fundInfo.Rows[0]["is_authorized"].ToString() == "1" ? true : false;

                #endregion

                #region Nav

                dtpNavStartDate.ShowCheckBox = true;
                dtpNavStartDate.Checked = false;
                dtpNavStartDate.Value = DateTime.Parse(lblInceptionDate.Text);

                dtpNavEndDate.ShowCheckBox = true;
                dtpNavEndDate.Checked = false;

                lblNavFrequency.Text = navFreq;

                LoadNavGrid(null, null);

                #endregion
            }

        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            string fundShortName = txtFundShortName.Text.Trim();
            int? strategyId = int.Parse(cmbStrategy.SelectedValue.ToString());
            int? subStrategyId = int.Parse(cmbSubStrategy.SelectedValue.ToString());
            string parentFundId = txtParentFund.Text.Trim();
            parentFundId = (string.IsNullOrEmpty(parentFundId) ? null : parentFundId);
            string navFrequency = cmbNavFrequency.SelectedText.ToString();
            sbyte isAuthorized = (sbyte)(chkAuthorized.Checked ? 1 : 0);
            sbyte isRanking = (sbyte)(chkRanking.Checked ? 1 : 0);
            sbyte isRating = (sbyte)(chkRating.Checked ? 1 : 0);

            DataAccess.Set_dm_fund_information(FundId, fundShortName, strategyId, subStrategyId, parentFundId, navFrequency, isAuthorized, isRanking, isRating, UserId);

            MessageBox.Show("所有更新顺利存盘", "提示信息", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private Dictionary<int, string> LoadSubStrategy(int strategyId)
        {
            Dictionary<int, string> d;
            int[] keysToSelect = null;

            switch (cmbStrategy.SelectedValue)
            {
                case 1:
                    keysToSelect = new int[] { 1010, 1020, 1030, -1 };
                    break;
                case 2:
                    keysToSelect = new int[] { 2010, -1 };
                    break;
                case 3:
                    keysToSelect = new int[] { 3010, 3020, 3030, 3040, 3050, -1 };
                    break;
                case 4:
                    keysToSelect = new int[] { 4010, 4020, 4030, 4040, -1 };
                    break;
                case 5:
                    keysToSelect = new int[] { 5010, 5020, 5030, -1 };
                    break;
                case 6:
                    keysToSelect = new int[] { 6010, 6020, 6030, -1 };
                    break;
                case 7:
                    keysToSelect = new int[] { 7010, 7020, -1 };
                    break;
                case 8:
                    keysToSelect = new int[] { 8010, 8020, -1 };
                    break;
                default:
                    keysToSelect = new int[] { -1 };
                    break;
            }

            d = UIConstants.HedgeFundSubStrategy.Where(kvp => keysToSelect.Contains(kvp.Key)).ToDictionary(kvp => kvp.Key, kvp => kvp.Value);

            return d;
        }


        /// <summary>
        /// 二级策略选项跟随改变
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbStrategy_SelectedIndexChanged(object sender, EventArgs e)
        {
            // 避免初始化绑定数据源时触发
            if (cmbStrategy.ContainsFocus)
            {
                int strategyId = int.Parse(cmbStrategy.SelectedValue.ToString());

                cmbSubStrategy.DataSource = new BindingSource(LoadSubStrategy(strategyId), null);
                cmbSubStrategy.DisplayMember = "Value";
                cmbSubStrategy.ValueMember = "Key";

            }
        }

        private void lblCompany_Click(object sender, EventArgs e)
        {
            string companyId = fundInfo.Rows[0]["advisor_id"].ToString();

            if (companyId.Length == 10)
            {
                UICompany frm = new UICompany(companyId, UserId);
                frm.ShowDialog();
            }
        }

        private void LoadNavGrid(DateTime? startDate, DateTime? endDate)
        {
            tbNav = DataAccess.Get_dm_nav(fundId, null, null, startDate, endDate);
            
            grdNav.DataSource = tbNav;
            grdNav.Columns["fund_id"].Visible = false;
            grdNav.Columns["provider_id"].Visible = false;
            grdNav.Columns["provider_name"].HeaderText = "数据源";
            grdNav.Columns["external_id"].HeaderText = "外部ID";
            grdNav.Columns["price_date"].HeaderText = "净值日期";
            grdNav.Columns["nav"].HeaderText = "净值";
            grdNav.Columns["cumulative_nav_withdrawal"].HeaderText = "现金分红累计净值";
            grdNav.Columns["cumulative_nav"].HeaderText = "分红再投累计净值";
            grdNav.Columns["cumulative_nav"].DefaultCellStyle.BackColor = Color.AntiqueWhite;
            grdNav.Columns["isvalid"].HeaderText = "状态";
            grdNav.Columns["creatorid"].Visible = false;
            grdNav.Columns["creator_name"].HeaderText = "创建";
            grdNav.Columns["createtime"].HeaderText = "创建日期";
            grdNav.Columns["updaterid"].Visible = false;
            grdNav.Columns["updater_name"].HeaderText = "更新";
            grdNav.Columns["updatetime"].HeaderText = "更新日期";

            grdNav.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            grdNav.RowHeadersVisible = false;

        }

        private void btnDrawNavLineChart_Click(object sender, EventArgs e)
        {
            NavChart chart = new NavChart(FundId, tbNav);
            chart.Show();
        }

        private void grdNav_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            //int rowIndex = e.RowIndex;
            //int columnIndex = e.ColumnIndex;

            //if (rowIndex < 0 || columnIndex < 0) return;

            //DataGridViewRow row = grdNav.Rows[rowIndex];

            //if(row == null) return;


        }

        private void btnAddNav_Click(object sender, EventArgs e)
        {

        }


    }
}
